# coding: utf-8
import datetime
import autopath
from os.path import join
import subprocess
from copy import copy
from bson.objectid import ObjectId
import logging
import ldb

from eoleaaf.aaf import reset_db
from eoleaaf.util import db
from eoleaaf.parseaaf import main
from eoleaaf.parsedelta import main as main_delta
from eoleaaf.sambatranslator import SambaToplevel, SambaSplit, erase_db, verify_indexed
verify_indexed()

# get BASEDN and AD_REALM
exec(file('/etc/eole/samba4-vars.conf').read())

data = join(autopath.HERE, 'data', '20170704', 'updategroup')

data_complet = join(data, 'aaf-complet')
data_delta = join(data, 'aaf-delta')

def setup_function(module):
    reset_db()
    erase_db()


etablissement = {
    u"ENTStructureJointure": "2516",
    u"ENTStructureUAI": "00000002",
    u"ENTEtablissementUAI": "00000002",
    u"ENTStructureSIREN": "17450234560290",
    u"ENTStructureNomCourant": "D.E.E.E.-DU ac-test",
    u"ENTStructureTypeStruct": "INSPECTIONS ACADEMIQUES",
    u"ENTEtablissementMinistereTutelle": "MINISTERE DE L'EDUCATION NATIONALE",
    u"ENTEtablissementContrat": "PU",
    u'FieldActionType': 'DELTACREATE_DONE',
    u'FieldModifiedStatus': 1,
    u"postOfficeBox": "608",
    u"street": "RUE CITE",
    u"postalCode": "18013",
    u"l": "DIJON CEDEX",
    u"telephoneNumber": "+33 211 25 687  00",
    u"facsimileTelephoneNumber": "+33 2 125899 20 01",
    u"ENTEtablissementStructRattachFctl": '',
    u"ENTEtablissementBassin": "BASSIN DU TEST",
    u"ENTServAcAcademie": "ac test",
    u"ENTStructureClasses": '',
    u"ENTStructureGroupes": [{'class': '4E3', 'group': '4E3_A'}, {'class': '4E3', 'group': '4E3_B'}],
}

etablissement2 = {
    u"ENTStructureJointure": "2515",
    u"ENTStructureUAI": "00000001",
    u"ENTEtablissementUAI": "00000001",
    u"ENTStructureSIREN": "17450234560289",
    u"ENTStructureNomCourant": "D.E.E.E.-DU ac-test2",
    u"ENTStructureTypeStruct": "INSPECTIONS ACADEMIQUES",
    u"ENTEtablissementMinistereTutelle": "MINISTERE DE L'EDUCATION NATIONALE",
    u"ENTEtablissementContrat": "PU",
    u'FieldActionType': 'DELTAUPDATE_DONE',
    u'FieldModifiedStatus': 1,
    u"postOfficeBox": "608",
    u"street": "RUE CITE",
    u"postalCode": "18013",
    u"l": "DIJON CEDEX",
    u"telephoneNumber": "+33 211 25 687  00",
    u"facsimileTelephoneNumber": "+33 2 125899 20 01",
    u"ENTEtablissementStructRattachFctl": '',
    u"ENTEtablissementBassin": "BASSIN DU TEST",
    u"ENTServAcAcademie": "ac test",
    u"ENTStructureClasses": '',
    u"ENTStructureGroupes": [{'class': '4E3', 'group': '4E3_A'}, {'class': '4E3', 'group': '4E3_B'}],
}

etablissements = {
        u'2516': etablissement,
        u'2515': etablissement2,
        }


etablissement_ad = {
    'dn': ['CN=00000001,CN=Users,{}'.format(BASEDN)],
    'objectClass': ['top', 'group'],
    'cn': ['00000001'],
    'description': ['Groupe D.E.E.E.-DU ac-test2'],
    'instanceType': ['4'],
    'info': ['e2515'],
    'name': ['00000001'],
    'sAMAccountName': ['00000001'],
    'groupType': ['-2147483646'],
    'objectCategory': ['CN=Group,CN=Schema,CN=Configuration,{}'.format(BASEDN)],
    'member': set(['CN=albertine.delacourt02,CN=Users,{}'.format(BASEDN), 'CN=albertine.delacourt03,CN=Users,{}'.format(BASEDN), 'CN=albertine.delacourt04,CN=Users,{}'.format(BASEDN), 'CN=ghislaine.delmare01,CN=Users,{}'.format(BASEDN)]),
    'distinguishedName': ['CN=00000001,CN=Users,{}'.format(BASEDN)],
}
etablissement_ad_split = copy(etablissement_ad)
etablissement_ad_split['dn'] = ['CN=00000001,OU=00000001,{}'.format(BASEDN)]
etablissement_ad_split['distinguishedName'] = etablissement_ad_split['dn']
etablissement_ad_split['member'] = set(['CN=albertine.delacourt02,OU=00000001,{}'.format(BASEDN), 'CN=albertine.delacourt03,OU=00000001,{}'.format(BASEDN), 'CN=albertine.delacourt04,OU=00000001,{}'.format(BASEDN), 'CN=ghislaine.delmare01,OU=00000001,{}'.format(BASEDN)])

etablissement_ad2 = {
    'dn': ['CN=00000002,CN=Users,{}'.format(BASEDN)],
    'objectClass': ['top', 'group'],
    'cn': ['00000002'],
    'description': ['Groupe D.E.E.E.-DU ac-test'],
    'instanceType': ['4'],
    'info': ['e2516'],
    'name': ['00000002'],
    'sAMAccountName': ['00000002'],
    'groupType': ['-2147483646'],
    'objectCategory': ['CN=Group,CN=Schema,CN=Configuration,{}'.format(BASEDN)],
    'distinguishedName': ['CN=00000002,CN=Users,{}'.format(BASEDN)],
}
etablissement_ad2_split = copy(etablissement_ad2)
etablissement_ad2_split['dn'] = ['CN=00000002,OU=00000002,{}'.format(BASEDN)]
etablissement_ad2_split['distinguishedName'] = etablissement_ad2_split['dn']

etablissements_ad_toplevel = {
        '2515': etablissement_ad,
        '2516': etablissement_ad2
        }

etablissements_ad_split = {
        '2515': etablissement_ad_split,
        '2516': etablissement_ad2_split
        }

def test_etablissement():
    for mode, etablissements_ad, SambaExport in [('toplevel', etablissements_ad_toplevel, SambaToplevel), ('split', etablissements_ad_split, SambaSplit)]:
        setup_function(None)
        keys = set(etablissement.keys()) | set([u"_id", u'FieldFromXMLImportActionDate', u'id', 'FieldToLDIFFExportActionDate'])
        log = logging.getLogger(__name__)
        main(data_complet, False, close=False)
        samba_import = SambaExport(log, debug=True)
        samba_import.run()
        main_delta(data_delta, close=False)
        samba_import = SambaExport(log, debug=True)
        samba_import.run()
        result = db.fetchall('etablissement')
        list_etablissement = etablissements.keys()
        for res in result:
            idx = res['id']
            if idx not in etablissements:
                continue
            print "etablissement", idx, 'en mode', mode
            list_etablissement.remove(idx)
            assert keys == set(res.keys())
            for key in etablissement.keys():
                assert res[key] == etablissements[idx][key], u"la valeur correspondant à : {} est différente".format(key)
            # we test what is in the AD
            filter_ = "(&(sAMAccountName=%s)(objectCategory=%s,%s))" % (ldb.binary_encode(res['ENTEtablissementUAI']),
                      "CN=Group,CN=Schema,CN=Configuration", samba_import.samdb.domain_dn())
            target = samba_import.samdb.search(base=samba_import.samdb.domain_dn(), scope=ldb.SCOPE_SUBTREE,
                                 expression=filter_)
            query_result = list(target)
            assert len(query_result) == 1
            list_keys = etablissements_ad[idx].keys()
            for key, value in query_result[0].items():
                values = []
                if key in ['whenCreated', 'whenChanged', 'objectGUID', 'objectSid', 'accountExpires', 'sAMAccountType', 'pwdLastSet', 'uSNChanged', 'uSNCreated']:
                    continue
                list_keys.remove(key)
                if isinstance(value, ldb.Dn):
                    values.append(str(value))
                else:
                    for i in value:
                        values.append(i)
                if key == 'member':
                    values = set(values)
                assert values == etablissements_ad[idx][key], u"valeurs différentes pour la clef : {}".format(key)
            assert not list_keys
        assert list_etablissement == []
